/*
** ADM-XRC Series Driver
**
** (C) Copyright 2001-2004 Alpha Data
**
** info.c - IOCTL handlers for returning information about a card.
**
** ----------------------------------------------------------------------------
*/

#include "platform.h"
#include "device.h"
#include "ioctls.h"
#include "info.h"
#include "driver_ver.h"
#include <admxrc2/geoginf.h>

OS_STATUS
ADMXRC_GetBankInfo(
  DeviceContext* pDevCtx,
  OS_IO_REQUEST* pReq,
  ADMXRC_IOCTLS_GETBANKINFO* pIoctl,
  unsigned long inSize,
  unsigned long outSize)
{
  unsigned long index;

  if (pIoctl == NULL || inSize != sizeof(pIoctl->in) || outSize != sizeof(pIoctl->out)) {
    return OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  }
  
  index = pIoctl->in.index;
  if (index >= pDevCtx->info.cardInfo.numRamBank) {
    return OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  }
  
  pIoctl->out.type = pDevCtx->info.bankInfo[index].type;
  pIoctl->out.width = pDevCtx->info.bankInfo[index].width;
  pIoctl->out.size = pDevCtx->info.bankInfo[index].size;
  pIoctl->out.fitted = pDevCtx->info.bankInfo[index].fitted;

  pReq->status.length = sizeof(pIoctl->out);

  return OS_STATUS_SUCCESS;
}

OS_STATUS
ADMXRC_GetGeographicalInfo(
  DeviceContext* pDevCtx,
  OS_IO_REQUEST* pReq,
  ADMXRC_IOCTLS_GETGEOGRAPHICALINFO* pIoctl,
  unsigned long outSize)
{
  if (pIoctl == NULL || outSize != sizeof(pIoctl->out)) {
    return OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  }

  pIoctl->out.type = ADMXRC2_GEOGRAPHICAL_PCI;
  pIoctl->out.info.pci.domain = pDevCtx->slotId.pci.domain;
  pIoctl->out.info.pci.bus = pDevCtx->slotId.pci.bus;
  pIoctl->out.info.pci.slot = pDevCtx->slotId.pci.slot;
  pIoctl->out.info.pci.function = pDevCtx->slotId.pci.function;

  pReq->status.length = sizeof(pIoctl->out);

  return OS_STATUS_SUCCESS;
}

OS_STATUS
ADMXRC_GetCardInfo2(
  DeviceContext* pDevCtx,
  OS_IO_REQUEST* pReq,
  ADMXRC_IOCTLS_GETCARDINFO2* pIoctl,
  unsigned long outSize)
{
  if (pIoctl == NULL || outSize != sizeof(pIoctl->out)) {
    return OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  }

  pIoctl->out.cardId = pDevCtx->info.cardInfo.cardId;
  pIoctl->out.serialNum = pDevCtx->info.cardInfo.serialNum;
  pIoctl->out.boardType = pDevCtx->info.cardInfo.boardType;
  pIoctl->out.fpgaType = pDevCtx->info.cardInfo.fpgaType;
  pIoctl->out.numClock = pDevCtx->info.cardInfo.numClock;
  pIoctl->out.numDmaChan = pDevCtx->info.cardInfo.numDmaChan;
  pIoctl->out.numRamBank = pDevCtx->info.cardInfo.numRamBank;
  pIoctl->out.numSpace = pDevCtx->info.cardInfo.numSpace;
  pIoctl->out.ramBanksFitted = pDevCtx->info.cardInfo.ramBanksFitted;
  pIoctl->out.boardRevision = pDevCtx->info.cardInfo.boardRevision;
  pIoctl->out.logicRevision = pDevCtx->info.cardInfo.logicRevision;

  pReq->status.length = sizeof(pIoctl->out);

  return OS_STATUS_SUCCESS;
}

OS_STATUS
ADMXRC_GetSpaceInfo(
  DeviceContext* pDevCtx,
  ClientContext* pClCtx,
  OS_IO_REQUEST* pReq,
  ADMXRC_IOCTLS_GETSPACEINFO* pIoctl,
  unsigned long inSize,
  unsigned long outSize)
{
  unsigned long index;

  if (pIoctl == NULL || inSize != sizeof(pIoctl->in) || outSize != sizeof(pIoctl->out)) {
    return OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  }
  
  index = pIoctl->in.index;
  if (index >= pDevCtx->info.cardInfo.numSpace) {
    return OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  }
  pIoctl->out.virtualBase = pClCtx->spaceInfo[index].virtualBase;
  pIoctl->out.virtualSize = pClCtx->spaceInfo[index].virtualSize;
  pIoctl->out.physicalBase = pDevCtx->info.spaceInfo[index].physicalBase;
  pIoctl->out.localBase = pDevCtx->info.spaceInfo[index].localBase;
  pIoctl->out.localSize = pDevCtx->info.spaceInfo[index].localSize;

  pReq->status.length = sizeof(pIoctl->out);

  return OS_STATUS_SUCCESS;
}

OS_STATUS 
ADMXRC_GetVersionInfo(
  DeviceContext* pDevCtx,
  OS_IO_REQUEST* pReq,
  ADMXRC_IOCTLS_GETVERSIONINFO* pIoctl,
  unsigned long outSize)
{
  if (pIoctl == NULL || outSize != sizeof(pIoctl->out)) {
    return OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  }
  
  pIoctl->out.major = ADMXRC_DRIVER_MAJORVER;
  pIoctl->out.minor = ADMXRC_DRIVER_MINORVER;

  pReq->status.length = sizeof(pIoctl->out);
  
  return OS_STATUS_SUCCESS;
}

