/*
** ADM-XRC Series Driver
**
** (C) Copright 2001-2005 Alpha Data
**
** methods_flash.c - Model-specific routines for accessing Flash memory.
*/

#include "drvparam.h"
#include "flash.h"
#include "methods.h"

MethodResult
methodFlashPlxbased(
  DeviceContext* pDevCtx,
  os_boolean     write,
  os_uint32      location,
  os_uint16*     value)
{
  os_uint8  page = (os_uint8) (location >> 21);
  os_uint8* flash = ((os_uint8*) pDevCtx->hw.control.plxbased) + 0x200000U;

  os_dbgprint(9, ("methodFlashPlxbased: write=%ld location=%lx *value=%lx\n",
		  (unsigned long) write, (unsigned long) location, (unsigned long) *value));
  
  if (pDevCtx->devId.pci.subDevice == PCI_SUBDEVICE_ADATA_ADMXRC2 ||
      pDevCtx->devId.pci.subDevice == PCI_SUBDEVICE_ADATA_ADPWRC2 ||
      pDevCtx->devId.pci.subDevice == PCI_SUBDEVICE_ADATA_ADPDRC2) {
    /* Set Flash page register on ADM-XRCII/ADP-WRCII/ADP-DRCII only */
    os_write_register_le8(&pDevCtx->hw.control.plxbased->variant.admxrc2.FLPAGE, page & 0x7fU);
    os_read_register_le8(&pDevCtx->hw.control.plxbased->variant.admxrc2.FLPAGE);
  } else {
    if (page) {
      return MethodResult_InvalidLocation;
    }
  }
  
  location &= 0x1fffffU;
  if (write) {
    os_write_register_le8(flash + location, (os_uint8) *value);
    os_read_register_le8(&pDevCtx->hw.control.plxbased->variant.admxrc2.FLPAGE);
  } else {
    *value = (os_uint16) os_read_register_le8(flash + location);
  }
  
  return MethodResult_Success;
}

MethodResult
methodFlashXrc2l(
  DeviceContext* pDevCtx,
  os_boolean     write,
  os_uint32      location,
  os_uint16*     value)
{
  return MethodResult_NotPresent;
}

MethodResult
methodFlashXpl(
  DeviceContext* pDevCtx,
  os_boolean     write,
  os_uint32      location,
  os_uint16*     value)
{
  XPLSPACE1REGS* regs;
  os_uint32      flctl, fldata;
  
  os_dbgprint(9, ("methodFlashXpl: write=%ld location=%lx *value=%lx\n",
		  (unsigned long) write, (unsigned long) location, (unsigned long) *value));
  
  regs = pDevCtx->hw.control.xpl;
  
  if (write) {
    /* Drive fl_a, fl_sel_n, fl_oe_n, fl_we_n, assert fl_sel_n */
    flctl = (location & 0x01ffffffU) | (0x1fU << 25);
    os_write_register_le32(&regs->FLCTL, flctl);
    /* Drive fl_d (data to Flash) */
    fldata = (*value & 0xffU) | 0x100U;
    os_write_register_le32(&regs->FLDATA, fldata);
    os_read_register_le32(&regs->FLCTL);
    /* Assert fl_we_n (write enable for Flash) */
    flctl |= (0x1U << 31);
    os_write_register_le32(&regs->FLCTL, flctl);
    os_read_register_le32(&regs->FLCTL);
    /* Deassert fl_we_n */
    flctl &= ~(0x1U << 31);
    os_write_register_le32(&regs->FLCTL, flctl);
    os_read_register_le32(&regs->FLCTL);
    /* Stop driving fl_d (data to Flash) */
    fldata &= ~0x100U;
    os_write_register_le32(&regs->FLDATA, fldata);
    /* Stop driving fl_a, fl_sel_n, fl_oe_n, fl_we_n, assert fl_sel_n */
    flctl &= ~(0x1fU << 25);
    os_write_register_le32(&regs->FLCTL, flctl);
    os_read_register_le32(&regs->FLCTL);
  } else {
    /* Drive fl_a, fl_sel_n, fl_oe_n, fl_we_n, assert fl_sel_n */
    flctl = (location & 0x01ffffffU) | (0x1fU << 25);
    os_write_register_le32(&regs->FLCTL, flctl);
    os_read_register_le32(&regs->FLCTL);
    /* Assert fl_oe_n (output enable for Flash) */
    flctl |= (0x1U << 30);
    os_write_register_le32(&regs->FLCTL, flctl);
    os_read_register_le32(&regs->FLCTL);
    /* Read fl_d (data from Flash) */
    *value = (os_uint8) os_read_register_le32(&regs->FLDATA);
    /* Stop driving fl_a, fl_sel_n, fl_oe_n, fl_we_n, assert fl_sel_n and deassert fl_oe_n */
    flctl &= ~(0x3fU << 25);
    os_write_register_le32(&regs->FLCTL, flctl);
    os_read_register_le32(&regs->FLCTL);
    os_dbgprint(9, ("methodFlashXpl: *value=%lx\n", (unsigned long)*value));
  
  }
  
  return MethodResult_Success;
}

MethodResult
methodFlashXrc2n(
  DeviceContext* pDevCtx,
  os_boolean     write,
  os_uint32      location,
  os_uint16*     value)
{
  CONTROLWRC2* regs;
  
  os_dbgprint(9, ("methodFlashXrc2n: write=%ld location=%lx *value=%lx\n",
		  (unsigned long) write, (unsigned long) location, (unsigned long) *value));
  
  regs = pDevCtx->hw.control.wrc2;
  
  /* Set up the address */
  os_write_register_le32(&regs->FLADDR, location);
  os_read_register_le32(&regs->FLADDR);
  
  if (write) {
    /* Write the value */
    os_write_register_le8(&regs->FLDATA, (os_uint8) *value);
  } else {
    /* Read the value */
    *value = (os_uint16) os_read_register_le8(&regs->FLDATA);
  }
  
  return MethodResult_Success;
}

MethodResult
methodFlashXprev0(
  DeviceContext* pDevCtx,
  os_boolean     write,
  os_uint32      location,
  os_uint16*     value)
{
  return MethodResult_NotPresent;
}

MethodResult
methodFlashXprev1(
  DeviceContext* pDevCtx,
  os_boolean     write,
  os_uint32      location,
  os_uint16*     value)
{
  XPSPACE1REGS* regs;
  os_uint32 flctl, fldata;
  os_uint32 val32;
  
  os_dbgprint(9, ("methodFlashXprev1: write=%ld location=%lx *value=%lx\n",
		  (unsigned long) write, (unsigned long) location, (unsigned long) *value));
  
  regs = pDevCtx->hw.control.xp;
  
  fldata = pDevCtx->shadow.regs.xp.fldata;
  
  /* Drive fl_a, fl_sel_n, fl_oe_n, fl_we_n, assert fl_sel_n */
  flctl = ((location & 0x1fffffeU) >> 1) | (0x1fU << 25);
  os_write_register_le32(&regs->FLCTL, flctl);
  val32 = os_read_register_le32(&regs->FLCTL);
  
  if (write) {
    /* Drive fl_d */
    fldata |= (*value | 0x10000U);
    os_write_register_le32(&regs->FLDATA, fldata);
    val32 = os_read_register_le32(&regs->FLDATA);
    
    /* Assert fl_we_n (write enable for Flash) */
    flctl |= (0x1U << 31);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Deassert fl_we_n */
    flctl &= ~(0x1U << 31);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Deassert fl_sel_n */
    flctl &= ~(0x1U << 29);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
  } else {
    /* Assert fl_oe_n */
    flctl |= (0x1U << 30);
    os_write_register_le32(&regs->FLCTL, flctl);
    
    /* Read fl_d */
    *value = (os_uint16) os_read_register_le32(&regs->FLDATA);
    
    /* Deassert fl_oe_n */
    flctl &= ~(0x1U << 30);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Deassert fl_sel_n */
    flctl &= ~(0x1U << 29);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    os_dbgprint(9, ("methodFlashXprev1: *value=0x%lx\n",
		    (unsigned long) *value));
  }
  
  /* Stop driving fl_a, fl_sel_n, fl_oe_n, fl_we_n */
  os_write_register_le32(&regs->FLCTL, 0);
  os_read_register_le32(&regs->FLCTL);
  
  /* Stop driving fl_d */
  fldata &= ~0x10000U;
  os_write_register_le32(&regs->FLDATA, fldata);
  os_read_register_le32(&regs->FLDATA);
  
  return MethodResult_Success;
}

MethodResult
methodFlashXrc4ls(
  DeviceContext* pDevCtx,
  os_boolean     write,
  os_uint32      location,
  os_uint16*     value)
{
  CONTROLXRC4LS* regs;
  os_uint32 flctl, flpage;
  os_uint32 val32;

  os_dbgprint(9, ("methodFlashXrc4ls: write=%ld location=%lx *value=%lx\n",
		  (unsigned long) write, (unsigned long) location, (unsigned long) *value));

  regs = pDevCtx->hw.control.xrc4ls;

  flpage = location & 0x1fe0000U;
  os_write_register_le32(&regs->FLPAGE, flpage);
  val32 = os_read_register_le32(&regs->FLPAGE);

  if (write) {
    /* Select mode 0, set fl_a, Drive fl_a, fl_d, fl_ce_n, fl_oe_n, fl_we_n, assert fl_ce_n */
    flctl = (location & 0x1fffeU) | (0x7U << 24) | (0x1U << 28);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Set fl_d */
    os_write_register_le32(&regs->FLDATA, *value);
    val32 = os_read_register_le32(&regs->FLDATA);
    
    /* Assert fl_we_n */
    flctl |= (0x1U << 30);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Deassert fl_we_n */
    flctl &= ~(0x1U << 30);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Deassert fl_ce_n */
    flctl &= ~(0x1U << 28);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Stop driving fl_a, fl_d, fl_ce_n, fl_oe_n, fl_we_n */
    flctl &= ~(0x7U << 24);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
  } else {
    /* Select mode 0, set fl_a, Drive fl_a, fl_ce_n, fl_oe_n, fl_we_n, assert fl_ce_n */
    flctl = (location & 0x1fffeU) | (0x3U << 25) | (0x1U << 28);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Assert fl_oe_n */
    flctl |= (0x1U << 29);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Read fl_d */
    *value = (os_uint16) os_read_register_le32(&regs->FLDATA);
    
    /* Deassert fl_oe_n */
    flctl &= ~(0x1U << 29);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Deassert fl_ce_n */
    flctl &= ~(0x1U << 28);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Stop driving fl_a, fl_ce_n, fl_oe_n, fl_we_n */
    flctl &= ~(0x3U << 25);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    os_dbgprint(9, ("methodFlashXrc4ls: *value=%lx\n",
		    (unsigned long) *value));
  }

  return MethodResult_Success;
}

MethodResult
methodFlashXrc4fx(
  DeviceContext* pDevCtx,
  os_boolean     write,
  os_uint32      location,
  os_uint16*     value)
{
  XRC4FXSPACE1REGS* regs;
  os_uint32 flctl, flpage;
  os_uint32 val32;
  
  os_dbgprint(9, ("methodFlashXrc4fx: write=%ld location=%lx *value=%lx\n",
		  (unsigned long) write, (unsigned long) location, (unsigned long) *value));
  
  regs = pDevCtx->hw.control.xrc4fx;
  
  flpage = location & 0x3fe0000U;
  os_write_register_le32(&regs->FLPAGE, flpage);
  val32 = os_read_register_le32(&regs->FLPAGE);
  
  if (write) {
    /* Select mode 0, set fl_a, Drive fl_a, fl_d, fl_ce_n, fl_oe_n, fl_we_n, assert fl_ce_n */
    flctl = (location & 0x1fffeU) | (0x7U << 24) | (0x1U << 28);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Set fl_d */
    os_write_register_le32(&regs->FLDATA, *value);
    val32 = os_read_register_le32(&regs->FLDATA);
    
    /* Assert fl_we_n */
    flctl |= (0x1U << 30);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Deassert fl_we_n */
    flctl &= ~(0x1U << 30);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Deassert fl_ce_n */
    flctl &= ~(0x1U << 28);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Stop driving fl_a, fl_d, fl_ce_n, fl_oe_n, fl_we_n */
    flctl &= ~(0x7U << 24);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
  } else {
    /* Select mode 0, set fl_a, Drive fl_a, fl_ce_n, fl_oe_n, fl_we_n, assert fl_ce_n */
    flctl = (location & 0x1fffeU) | (0x3U << 25) | (0x1U << 28);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Assert fl_oe_n */
    flctl |= (0x1U << 29);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Read fl_d */
    *value = (os_uint16) os_read_register_le32(&regs->FLDATA);
    
    /* Deassert fl_oe_n */
    flctl &= ~(0x1U << 29);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Deassert fl_ce_n */
    flctl &= ~(0x1U << 28);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Stop driving fl_a, fl_ce_n, fl_oe_n, fl_we_n */
    flctl &= ~(0x3U << 25);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    os_dbgprint(9, ("methodFlashXrc4fx: *value=%lx\n",
		    (unsigned long) *value));
  }
  
  return MethodResult_Success;
}

MethodResult
methodFlashXrce4fx(
  DeviceContext* pDevCtx,
  os_boolean     write,
  os_uint32      location,
  os_uint16*     value)
{
  XRCE4FXSPACE1REGS* regs;
  os_uint32 flctl, flpage;
  os_uint32 val32;
  
  os_dbgprint(9, ("methodFlashXrce4fx: write=%ld location=%lx *value=%lx\n",
		  (unsigned long) write, (unsigned long) location, (unsigned long) *value));
  
  regs = pDevCtx->hw.control.xrce4fx;
  
  flpage = location & 0x1fe0000U;
  os_write_register_le32(&regs->FLPAGE, flpage);
  val32 = os_read_register_le32(&regs->FLPAGE);
  
  if (write) {
    /* Select mode 0, set fl_a, Drive fl_a, fl_d, fl_ce_n, fl_oe_n, fl_we_n, assert fl_ce_n */
    flctl = (location & 0x1fffeU) | (0x7U << 24) | (0x1U << 28);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Set fl_d */
    os_write_register_le32(&regs->FLDATA, *value);
    val32 = os_read_register_le32(&regs->FLDATA);
    
    /* Assert fl_we_n */
    flctl |= (0x1U << 30);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Deassert fl_we_n */
    flctl &= ~(0x1U << 30);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Deassert fl_ce_n */
    flctl &= ~(0x1U << 28);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Stop driving fl_a, fl_d, fl_ce_n, fl_oe_n, fl_we_n */
    flctl &= ~(0x7U << 24);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
  } else {
    /* Select mode 0, set fl_a, Drive fl_a, fl_ce_n, fl_oe_n, fl_we_n, assert fl_ce_n */
    flctl = (location & 0x1fffeU) | (0x3U << 25) | (0x1U << 28);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Assert fl_oe_n */
    flctl |= (0x1U << 29);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Read fl_d */
    *value = (os_uint16) os_read_register_le32(&regs->FLDATA);
    
    /* Deassert fl_oe_n */
    flctl &= ~(0x1U << 29);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Deassert fl_ce_n */
    flctl &= ~(0x1U << 28);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    /* Stop driving fl_a, fl_ce_n, fl_oe_n, fl_we_n */
    flctl &= ~(0x3U << 25);
    os_write_register_le32(&regs->FLCTL, flctl);
    val32 = os_read_register_le32(&regs->FLCTL);
    
    os_dbgprint(9, ("methodFlashXrce4fx: *value=%lx\n",
		    (unsigned long) *value));
  }
  
  return MethodResult_Success;
}

/*
** -----------------------------------------------------------------------
** Flash Reset Methods
** -----------------------------------------------------------------------
*/

MethodResult
methodFlashResetPlxbased(
  DeviceContext* pDevCtx)
{
  os_dbgprint(9, ("methodFlashResetPlxbased:\n"));

  return MethodResult_Success;
}

MethodResult
methodFlashResetXpl(
  DeviceContext* pDevCtx)
{
  os_uint32 fldata;
  XPLSPACE1REGS* regs;
  
  os_dbgprint(9, ("methodFlashResetXpl:\n"));
  
  regs = pDevCtx->hw.control.xpl;
  
  /* Drive fl_rst_n, assert fl_rst_n */
  fldata = (0x3U << 30);
  os_write_register_le32(&regs->FLDATA, fldata);
  os_read_register_le32(&regs->FLDATA);
  
  /* Deassert fl_rst_n */
  fldata = (0x1U << 30);
  os_write_register_le32(&regs->FLDATA, fldata);
  os_read_register_le32(&regs->FLDATA);

  /* Stop driving fl_rst_n */
  fldata = (0x0U << 30);
  os_write_register_le32(&regs->FLDATA, fldata);
  os_read_register_le32(&regs->FLDATA);

  /* Wait 20us or so for recovery from reset */
  os_delay_busy_us(20);
  
  return MethodResult_Success;
}

MethodResult
methodFlashResetXp(
  DeviceContext* pDevCtx)
{
  os_uint32 fldata;
  XPSPACE1REGS* regs;
  
  os_dbgprint(9, ("methodFlashResetXp:\n"));
  
  regs = pDevCtx->hw.control.xp;
  
  fldata = pDevCtx->shadow.regs.xp.fldata;
  
  /* Drive fl_rst_n, assert fl_rst_n */
  fldata |= (0x3U << 30);
  os_write_register_le32(&regs->FLDATA, fldata);
  os_read_register_le32(&regs->FLDATA);
  
  /* Deassert fl_rst_n */
  fldata &= ~(0x1U << 31);
  os_write_register_le32(&regs->FLDATA, fldata);
  os_read_register_le32(&regs->FLDATA);
  
  /* Stop driving fl_rst_n */
  fldata &= ~(0x1U << 30);
  os_write_register_le32(&regs->FLDATA, fldata);
  os_read_register_le32(&regs->FLDATA);
  
  /* Wait 20us or so for recovery from reset */
  os_delay_busy_us(20);
  
  return MethodResult_Success;
}

MethodResult
methodFlashResetXrc4ls(
  DeviceContext* pDevCtx)
{
  os_uint32 flctl;
  CONTROLXRC4LS* regs;

  os_dbgprint(9, ("methodFlashResetXrc4ls:\n"));

  regs = pDevCtx->hw.control.xrc4ls;
  
  flctl = pDevCtx->shadow.regs.xrc4ls.flctl;

  /* Drive fl_rst_n, assert fl_rst_n */
  flctl |= (0x1U << 27) | (0x1U << 31);
  os_write_register_le32(&regs->FLDATA, flctl);
  os_read_register_le32(&regs->FLDATA);

  /* Deassert fl_rst_n */
  flctl &= ~(0x1U << 31);
  os_write_register_le32(&regs->FLDATA, flctl);
  os_read_register_le32(&regs->FLDATA);

  /* Stop driving fl_rst_n */
  flctl &= ~(0x1U << 27);
  os_write_register_le32(&regs->FLDATA, flctl);
  os_read_register_le32(&regs->FLDATA);
  
  /* Wait 20us or so for recovery from reset */
  os_delay_busy_us(20);
  
  return MethodResult_Success;
}

MethodResult
methodFlashResetXrc4fx(
  DeviceContext* pDevCtx)
{
  os_uint32 flctl;
  XRC4FXSPACE1REGS* regs;
  
  os_dbgprint(9, ("methodFlashResetXrc4fx\n"));
  
  regs = pDevCtx->hw.control.xrc4fx;
  
  flctl = pDevCtx->shadow.regs.xrc4fx.flctl;
  
  /* Drive fl_rst_n, assert fl_rst_n */
  flctl |= (0x1U << 27) | (0x1U << 31);
  os_write_register_le32(&regs->FLCTL, flctl);
  os_read_register_le32(&regs->FLCTL);
  
  /* Deassert fl_rst_n */
  flctl &= ~(0x1U << 31);
  os_write_register_le32(&regs->FLCTL, flctl);
  os_read_register_le32(&regs->FLCTL);
  
  /* Stop driving fl_rst_n */
  flctl &= ~(0x1U << 27);
  os_write_register_le32(&regs->FLCTL, flctl);
  os_read_register_le32(&regs->FLCTL);
  
  os_delay_busy_us(20);
  
  return MethodResult_Success;
}

MethodResult
methodFlashResetXrce4fx(
  DeviceContext* pDevCtx)
{
  os_uint32 flctl;
  XRCE4FXSPACE1REGS* regs;
  
  os_dbgprint(9, ("methodFlashResetXrce4fx:\n"));
  
  regs = pDevCtx->hw.control.xrce4fx;
  
  flctl = pDevCtx->shadow.regs.xrce4fx.flctl;
  
  /* Drive fl_rst_n, assert fl_rst_n */
  flctl |= (0x1U << 27) | (0x1U << 31);
  os_write_register_le32(&regs->FLCTL, flctl);
  os_read_register_le32(&regs->FLCTL);
  
  /* Deassert fl_rst_n */
  flctl &= ~(0x1U << 31);
  os_write_register_le32(&regs->FLCTL, flctl);
  os_read_register_le32(&regs->FLCTL);
  
  /* Stop driving fl_rst_n */
  flctl &= ~(0x1U << 27);
  os_write_register_le32(&regs->FLCTL, flctl);
  os_read_register_le32(&regs->FLCTL);
  
  os_delay_busy_us(20);
  
  return MethodResult_Success;
}
