#ifndef _ADATA_XPLBRIDGE_H
#define _ADATA_XPLBRIDGE_H

/* ------------------------------------------------------------------------ */
/* ADM-XPL/XP/XPI Bridge                                                    */
/* ------------------------------------------------------------------------ */

#define NUM_XPLBRIDGE_DMA_CHANNEL (2)

/* Flags for NDL field of XPLDMADESC */
#define XPLDMADESC_LOCALSPACE	(0x0U << 0)
#define XPLDMADESC_PCISPACE		(0x1U << 0)
#define XPLDMADESC_ENDCHAIN		(0x1U << 1)
#define XPLDMADESC_INTERRUPT	(0x1U << 2)

/* Flags for SIZ field of XPLDMADESC */
#define XPLDMADESC_LOCALTOPCI	(0x1U << 31)
#define XPLDMADESC_PCITOLOCAL	(0x0U << 31)

/* An XPL bridge DMA descriptor */
typedef struct _XPLDMADESC {
  os_uint32     PAL;
  os_uint32     PAH;
  os_uint32     LA;
  os_uint32     SIZ;
  os_uint32     NDL;
  os_uint32     NDH;
  os_uint32     reserved1[2];
} os_declare_struct_packed XPLDMADESC;

typedef struct _XPLDMACON {
  os_uint32     PAL;
  os_uint32     PAH;
  os_uint32     LA;
  os_uint32     SIZ;
  os_uint32     NDL;
  os_uint32     NDH;
  os_uint32     CFG;
  os_uint32     CTL;
} os_declare_struct_packed XPLDMACON;

typedef struct _XPLBRIDGEREGS {
  os_uint32     LS0B;           /*  00 */
  os_uint32     LS0R;           /*  04 */
  os_uint32     LS0CFG;         /*  08 */
  os_uint32     reserved1[1];   /*  0C */
  os_uint32     LS1B;           /*  10 */
  os_uint32     LS1R;           /*  14 */
  os_uint32     LS1CFG;         /*  18 */
  os_uint32     reserved2[1];   /*  1C */
  os_uint32     LSTO;           /*  20 */
  os_uint32     LSTAT;          /*  24 */
  os_uint32     LCTL;           /*  28 */
  os_uint32     LERC;           /*  2C */
  os_uint32     LERA;           /*  30 */
  os_uint32     LICTL;          /*  34 */
  os_uint32     LISTAT;         /*  38 */
  os_uint32     reserved3[5];   /*  3C */
  os_uint32     PICTL;          /*  50 */
  os_uint32     PISTAT;         /*  54 */
  os_uint32     reserved4[10];  /*  58 */
  XPLDMACON     DMA[2];         /*  80 */
  os_uint32     LBRRD[4];       /*  C0 */
  os_uint32     LBRTM[4];       /*  D0 */
  os_uint32     LBRTV[4];       /*  E0 */
  os_uint32     LBRCSR;         /*  F0 */
  os_uint32     LBRA;           /*  F4 */
  os_uint32     reserved5[2];   /*  F8 */
} os_declare_struct_packed XPLBRIDGEREGS;

typedef struct _XPLSPACE1REGS {
  os_uint32     CLOCK;
  os_uint32     FLCTL;
  os_uint32     FLDATA;
  os_uint32     FPCTL;
  os_uint8      FPSMAP[4];
  os_uint32     FICTL;
  os_uint32     FISTAT;
  os_uint32     reserved1[1017];
} os_declare_struct_packed XPLSPACE1REGS;

typedef struct _XPSPACE1REGS {
  os_uint32     CLOCK;
  os_uint32     FLCTL;
  os_uint32     FLDATA;
  os_uint32     FPCTL;
  os_uint8      FPSMAP[4];
  os_uint32     FICTL;
  os_uint32     FISTAT;
  os_uint32     reserved1[1017];
} os_declare_struct_packed XPSPACE1REGS;

/* ------------------------------------------------------------------------ */
/* ADM-XRC-4FX Bridge                                                       */
/* ------------------------------------------------------------------------ */

#define NUM_XRC4FXBRIDGE_DMA_CHANNEL (4)

typedef struct _XRC4FXDMACON {
  os_uint32   PAL;
  os_uint32   PAH;
  os_uint32   LA;
  os_uint32   SIZ;
  os_uint32   NDL;
  os_uint32   NDH;
  os_uint32   CFG;
  os_uint32   CTL;
} os_declare_struct_packed XRC4FXDMACON;

typedef struct _XRC4FXBRIDGEREGS {
  os_uint32  LS0B;                   /* 0x00 */
  os_uint32  LS0R;                   /* 0x04 */
  os_uint32  LS0CFG;                 /* 0x08 */
  os_uint32  reserved1[1];   /* 0x0C */
  os_uint32  LS1B;                   /* 0x10 Bridge rev 1 */
  os_uint32  LS1R;                   /* 0x14 Bridge rev 1 */
  os_uint32  LS1CFG;                 /* 0x18 Bridge rev 1 */
  os_uint32  reserved2[1];   /* 0x1C */
  os_uint32  LSTO;                   /* 0x20 */
  os_uint32  LSTAT;                  /* 0x24 */
  os_uint32  LCTL;                   /* 0x28 */
  os_uint32  LERC;                   /* 0x2C */
  os_uint32  LERA;                   /* 0x30 */
  os_uint32  LICTL;                  /* 0x34 */
  os_uint32  LISTAT;                 /* 0x38 */
  os_uint32  reserved3[5];   /* 0x3C */
  os_uint32  PICTL;          /* 0x50 */
  os_uint32  PISTAT;         /* 0x54 */
  os_uint32  reserved4[10];  /* 0x58 */
  XRC4FXDMACON DMA[NUM_XRC4FXBRIDGE_DMA_CHANNEL]; /* 0x80 */
} os_declare_struct_packed XRC4FXBRIDGEREGS;

typedef struct _XRC4FXSPACE1REGS {
  os_uint32  reserved1[2];
  os_uint32  USERMGT;
  os_uint32  reserved2[1];
  os_uint32  LCLKCTL;
  os_uint32  LCLKSTAT1;
  os_uint32  LCLKSTAT2;
  os_uint32  LCLKSTAT3;
  os_uint8   FPSMAP[4];
  os_uint32  FPCTL;
  os_uint32  ICTL;
  os_uint32  ISTAT;
  os_uint32  PWRCTL;
  os_uint32  PWRSTAT;
  os_uint32  reserved3[50];
  os_uint32  FLCTL;
  os_uint32  FLPAGE;
  os_uint32  FLDATA;
  os_uint32  reserved4[13];
  os_uint8   FLBUF[64];
  os_uint32  reserved5[32];
  os_uint32  XRMID[17];
  os_uint32  reserved6[111];
  os_uint32  SYSMONCTL;
  os_uint32  reserved7[63];
  os_uint8   SYSMONBUF[0x100];
  os_uint8   I2C[0x200];
  os_uint8   reserved8[0x3F800];
  os_uint8   PARJTAG[0x20000];
  os_uint8   reserved9[0x20000];
  os_uint8   FLASH[0x40000];
  os_uint8   reserved10[0x40000];
} os_declare_struct_packed XRC4FXSPACE1REGS;

/* ------------------------------------------------------------------------ */
/* ADPE-XRC-4FX Bridge                                                      */
/* ------------------------------------------------------------------------ */

typedef struct _XRCE4FXSPACE1REGS {
  os_uint32  reserved1[2];
  os_uint32  LCLK;
  os_uint32  MCLK;
  os_uint32  reserved2;
  os_uint32  FPCTL;
  os_uint8   FPSMAP[4];
  os_uint32  reserved3;
  os_uint32  ICTL;
  os_uint32  ISTAT;
  os_uint32  PWRCTL;
  os_uint32  FLCTL;
  os_uint32  FLPAGE;
  os_uint32  FLDATA;
  os_uint32  reserved4[2];
  os_uint32  MONCTL;
  os_uint8   MONDATA[60];
} os_declare_struct_packed XRCE4FXSPACE1REGS;

/* ------------------------------------------------------------------------ */
/* ADM-XRC-5LX Bridge                                                       */
/* ------------------------------------------------------------------------ */

typedef struct _XRC5LXSPACE1REGS {
  os_uint32  reserved1[4];
  os_uint32  LCLKCTL;
  os_uint32  MCLKCTL;
  os_uint32  reserved2[2];
  os_uint8   FPSMAP[4];
  os_uint32  FPCTL;
  os_uint32  ICTL;
  os_uint32  ISTAT;
  os_uint32  PWRCTL;
  os_uint32  PWRSTAT;
  os_uint32  reserved3[50];
  os_uint32  FLCTL;
  os_uint32  FLPAGE;
  os_uint32  FLDATA;
  os_uint32  reserved4[13];
  os_uint8   FLBUF[64];
  os_uint32  reserved5[32];
  os_uint32  XRMID[17];
  os_uint32  reserved6[111];
  os_uint32  SYSMONCTL;
  os_uint32  reserved7[63];
  os_uint8   SYSMONBUF[0x100];
  os_uint8   I2C[0x200];
  os_uint8   reserved8[0x3F800];
  os_uint8   PARJTAG[0x20000];
  os_uint8   reserved9[0x20000];
  os_uint8   FLASH[0x40000];
  os_uint8   reserved10[0x40000];
} os_declare_struct_packed XRC5LXSPACE1REGS;

/* ------------------------------------------------------------------------ */
/* ADM-XRC-5LXA Bridge                                                      */
/* ------------------------------------------------------------------------ */

typedef struct _XRC5LXASPACE1REGS {
  os_uint32  reserved1[2];
  os_uint32  MCLKCTL;
  os_uint32  reserved2;
  os_uint32  LCLKCTL;
  os_uint32  LCLKSTAT1;
  os_uint32  LCLKSTAT2;
  os_uint32  LCLKSTAT3;
  os_uint8   FPSMAP[4];
  os_uint32  FPCTL;
  os_uint32  ICTL;
  os_uint32  ISTAT;
  os_uint32  PWRCTL;
  os_uint32  PWRSTAT;
  os_uint32  reserved3[50];
  os_uint32  FLCTL;
  os_uint32  FLPAGE;
  os_uint32  FLDATA;
  os_uint32  reserved4[13];
  os_uint8   FLBUF[64];
  os_uint32  reserved5[32];
  os_uint32  XRMID[17];
  os_uint32  reserved6[111];
  os_uint32  SYSMONCTL;
  os_uint32  reserved7[63];
  os_uint8   SYSMONBUF[0x100];
  os_uint8   I2C[0x200];
  os_uint8   reserved8[0x3F800];
  os_uint8   PARJTAG[0x20000];
  os_uint8   reserved9[0x20000];
  os_uint8   FLASH[0x40000];
  os_uint8   reserved10[0x40000];
} XRC5LXASPACE1REGS;

/* ------------------------------------------------------------------------ */
/* Board VPD blocks                                                         */
/* ------------------------------------------------------------------------ */

typedef struct _XPLCONFIGREV0 {
  os_uint16     version;
  os_uint16     boardType;
  os_uint32     modifications;
  os_uint32     refClockFreq;
  struct {
    os_uint8    order;
    os_uint8    width;
    os_uint16   typeMask;
  } os_declare_struct_packed bank[2];
  os_uint32     cardId;
  os_uint32     serialNumber;
  os_uint16     fpgaType;
  os_uint8      pcbRev;
  os_uint8      cpldRev;
} os_declare_struct_packed XPLCONFIGREV0;

typedef struct _XPCONFIGREV0 {
  os_uint16     version;
  os_uint16     boardType;
  os_uint32     modifications;
  os_uint32     refClockFreq;
  struct {
    os_uint8    order;
    os_uint8    reserved1;
  } os_declare_struct_packed zbt[4];
  struct {
    os_uint8    order;
    os_uint8    reserved1;
  } os_declare_struct_packed sdram[2];
  os_uint32     cardId;
  os_uint32     serialNumber;
  os_uint16     fpgaType;
  os_uint8      pcbRev;
  os_uint8      reserved;
} os_declare_struct_packed XPCONFIGREV0;

typedef struct _XPICONFIGREV0 {
  os_uint16     version;
  os_uint16     boardType;
  os_uint32     modifications;
  os_uint32     refClockFreq;
  struct {
    os_uint8    order;
    os_uint8    reserved1;
  } os_declare_struct_packed sdram[4];
  struct {
    os_uint8    order;
    os_uint8    reserved1;
  } os_declare_struct_packed sram[1];
  os_uint16     reserved1;
  os_uint32     cardId;
  os_uint32     serialNumber;
  os_uint16     fpgaType;
  os_uint8      pcbRev;
  os_uint8      reserved2;
} os_declare_struct_packed XPICONFIGREV0;

typedef struct _XRCE4FXCONFIGREV0 {
  os_uint16     version;
  os_uint16     boardType;
  os_uint32     modifications;
  os_uint8      stepping;
  os_uint8      reserved1[3];
  os_uint32     scd;
  os_uint32     refClk;
  os_uint32     lclkRef;
  os_uint32     mclkRef;
  os_uint32     reserved2;
  struct {
    os_uint8    order;
    os_uint8    reserved1;
  } os_declare_struct_packed sdram[4];
  os_uint32     cardId;
  os_uint32     serialNumber;
  os_uint16     fpgaType;
  os_uint8      pcbRev;
  os_uint8      cpldRev;
} os_declare_struct_packed XRCE4FXCONFIGREV0;

typedef struct _XRC4FXCONFIGREV0 {
  os_uint16     version;
  os_uint16     boardType;
  os_uint32     modifications;
  os_uint8      stepping;
  os_uint8      reserved1[3];
  os_uint32     scd;
  os_uint32     lclkRef;
  os_uint32     mclkARef;
  os_uint32     mclkBRef;
  os_uint32     reserved2;
  struct {
    os_uint8    order;
    os_uint8    reserved1;
  } os_declare_struct_packed sdram[4];
  os_uint32     cardId;
  os_uint32     serialNumber;
  os_uint16     fpgaType;
  os_uint8      pcbRev;
  os_uint8      cpldRev;
} os_declare_struct_packed XRC4FXCONFIGREV0;

typedef struct _XRC5LXCONFIGREV0 {
  os_uint16     version;
  os_uint16     boardType;
  os_uint32     modifications;
  os_uint8      stepping;
  os_uint8      reserved1[3];
  os_uint32     scd;
  os_uint32     refClk;
  os_uint32     lclkRef;
  os_uint32     mclkRef;
  os_uint32     reserved2;
  struct {
    os_uint8    order;
    os_uint8    reserved1;
  } os_declare_struct_packed sdram[4];
  os_uint32     cardId;
  os_uint32     serialNumber;
  os_uint16     fpgaType;
  os_uint8      pcbRev;
  os_uint8      cpldRev;
} os_declare_struct_packed XRC5LXCONFIGREV0;

typedef struct _XRC5T1CONFIGREV0 {
  os_uint16     version;
  os_uint16     boardType;
  os_uint32     modifications;
  os_uint8      stepping;
  os_uint8      reserved1[3];
  os_uint32     scd;
  os_uint32     lclkRef;
  os_uint32     mclkARef;
  os_uint32     mclkBRef;
  os_uint32     reserved2;
  struct {
    os_uint8    order;
    os_uint8    reserved1;
  } sdram[2];
  struct {
    os_uint8    order;
    os_uint8    reserved1;
  } ssram[1];
  os_uint16     reserved3;
  os_uint32     cardId;
  os_uint32     serialNumber;
  os_uint16     fpgaType;
  os_uint8      pcbRev;
  os_uint8      cpldRev;
} XRC5T1CONFIGREV0;

typedef struct _XRC5LXACONFIGREV0 {
  os_uint16     version;
  os_uint16     boardType;
  os_uint32     modifications;
  os_uint8      stepping;
  os_uint8      reserved1[3];
  os_uint32     scd;
  os_uint32     lclkRef;
  os_uint32     reserved2;
  os_uint32     mclkRef;
  os_uint32     reserved3;
  struct {
    os_uint8    order;
    os_uint8    reserved1;
  } sdram[4];
  os_uint32     cardId;
  os_uint32     serialNumber;
  os_uint16     fpgaType;
  os_uint8      pcbRev;
  os_uint8      cpldRev;
} XRC5LXACONFIGREV0;

typedef struct _XRCP4LXCONFIGREV0 {
  os_uint16     version;
  os_uint16     boardType;
  os_uint32     modifications;
  os_uint8      stepping;
  os_uint8      reserved1[3];
  os_uint32     scd;
  os_uint32     reserved2[4];
  struct {
    os_uint8    order;
    os_uint8    reserved1;
  } os_declare_struct_packed sdram[2];
  struct {
    os_uint8    order;
    os_uint8    reserved1;
  } os_declare_struct_packed zbt[2];
  os_uint32     cardId;
  os_uint32     serialNumber;
  os_uint16     fpgaType;
  os_uint8      pcbRev;
  os_uint8      cpldRev;
} os_declare_struct_packed XRCP4LXCONFIGREV0;

typedef struct _XRC5T2CONFIGREV0 {
  os_uint16     version;
  os_uint16     boardType;
  os_uint32     modifications;
  os_uint8      stepping;
  os_uint8      reserved1[3];
  os_uint32     scd;
  os_uint32     lclkRef;
  os_uint32     mclkARef;
  os_uint32     mclkBRef;
  os_uint32     reserved2;
  struct {
    os_uint8    order;
    os_uint8    reserved1;
  } sdram[4];
  struct {
    os_uint8    order;
    os_uint8    reserved1;
  } ssram[2];
  os_uint32     reserved3;
  os_uint32     cardId;
  os_uint32     serialNumber;
  os_uint16     fpgaType;
  os_uint8      pcbRev;
  os_uint8      cpldRev;
} XRC5T2CONFIGREV0;

#define VPD_ADDR_ADMAMC5A2 (0x1000000)

typedef struct _AMC5A2CONFIGREV0 {
  os_uint16     version;
  os_uint16     boardType;
  os_uint32     modifications;
  os_uint8      stepping;
  os_uint8      tempGrade;
  os_uint16     reserved1;
  os_uint32     scd;
  os_uint32     lclkRef;
  os_uint32     reserved2[3];
  struct {
    os_uint8    order;
    os_uint8    reserved1;
  } sdram[4];
  os_uint32     reserved3[2];
  os_uint32     cardId;
  os_uint32     serialNumber;
  os_uint16     fpgaType;
  os_uint8      pcbRev;
  os_uint8      cpldRev;
} AMC5A2CONFIGREV0;

typedef struct _XRC5TZCONFIGREV0 {
  os_uint16     version;
  os_uint16     boardType;
  os_uint32     modifications;
  os_uint8      stepping;
  os_uint8      tempGrade;
  os_uint8      reserved1[2];
  os_uint32     scd;
  os_uint32     lclkRef;
  os_uint32     mclkARef;
  os_uint32     mclkBRef;
  os_uint32     reserved2;
  struct {
    os_uint8    order;
    os_uint8    type;
  } zbt[6];
  os_uint32     reserved3;
  os_uint32     cardId;
  os_uint32     serialNumber;
  os_uint16     fpgaType;
  os_uint8      pcbRev;
  os_uint8      cpldRev;
} XRC5TZCONFIGREV0;

#endif
